###############################################################################
# Numerical illustration: toy example
# Cluster RCT with a small number of (equally-sized) large clusters,
# and a large number of (equally-sized) small clusters.
# Clusters are heterogeneous in means but homoskedastic.
# Mean and var of cluster size calibrated to match empirical literature.
###############################################################################

rm(list = ls())

###############################################################################
## Setup
###############################################################################

G <- 200
GL <- 10
Gs <- G-GL
ns <- 25
nL <- 100
n <- GL*nL+Gs*ns
piL <- nL*GL/n
nbar <- n/G
nVar <- GL*(nL-nbar)^2/G+Gs*(ns-nbar)^2/G

nbar
sqrt(nVar)

GL*nL/G+Gs*ns/G
sqrt(GL*Gs*(nL-ns)^2/G^2)

nVar/nbar^2

q <- 0.5

sigma2_alpha <- 1/2
sigma2_e <- 1/2
mus <- 0
muL <- 1
bs <- 0
bL <- 0

b <- piL*bL+(1-piL)*bs
b

###############################################################################
## True asymptotic variance
###############################################################################

V1 <- (sigma2_e + sigma2_alpha*(nL*piL+ns*(1-piL))) / (q*(1-q))
V2 <- piL*(1-piL)*(nL*(1-piL)+ns*piL)*q*(1-q)*( (muL-mus)/(q*(1-q)) + (bL-bs)/q)^2

V <- V1+V2
V

###############################################################################
## Probability limit of CR variance estimator
###############################################################################

Vcr <- V + piL*(1-piL)*(bL-bs)^2*(nL*(1-piL)+ns*piL)

sqrt(V/n)
sqrt(Vcr/n)

###############################################################################
## Naive variance 1: no heterogeneity
###############################################################################

NV1 <- (sigma2_e + sigma2_alpha*n/G) / (q*(1-q))

###############################################################################
## Naive variance 2: unequally sized clusters but no distributional heterogeneity
###############################################################################

NV2 <- V1

V
Vcr
NV1
NV2

###############################################################################
## Power function
###############################################################################

power <- function(beta,V) {1 - pnorm(beta/sqrt(V)+qnorm(0.975))+pnorm(beta/sqrt(V)-qnorm(0.975))}

###############################################################################
## MDEs for each variance
###############################################################################

bseq <- seq(-0.5,0.5,by=0.001)

beta <- power(bseq,V/n)
beta_cr <- power(bseq,Vcr/n)
beta_NV1 <- power(bseq,NV1/n)
beta_NV2 <- power(bseq,NV2/n)

MDE1 <- bseq[min(which(beta<=0.8))-1]
MDE2 <- bseq[max(which(beta<=0.8))+1]
MDE <- min(abs(MDE1),abs(MDE2))

MDE1_cr <- bseq[min(which(beta_cr<=0.8))-1]
MDE2_cr <- bseq[max(which(beta_cr<=0.8))+1]
MDE_cr <- min(abs(MDE1_cr),abs(MDE2_cr))

MDE1_NV1 <- bseq[min(which(beta_NV1<=0.8))-1]
MDE2_NV1 <- bseq[max(which(beta_NV1<=0.8))+1]
MDE_NV1 <- min(abs(MDE1_NV1),abs(MDE2_NV1))

MDE1_NV2 <- bseq[min(which(beta_NV2<=0.8))-1]
MDE2_NV2 <- bseq[max(which(beta_NV2<=0.8))+1]
MDE_NV2 <- min(abs(MDE1_NV2),abs(MDE2_NV2))

###############################################################################
## Figure 2
###############################################################################

pdf('powerplot3.pdf')
plot(NA,xlim=c(min(bseq),max(bseq)),ylim=c(0,1),xlab="Difference in means",ylab="Power",bty="L")
lines(bseq,beta_NV1,lwd=2,lty='dashed')
abline(h=0.8,lty='dotted')
abline(v=MDE_NV1,lty='dashed')
abline(v=-MDE_NV1,lty='dashed')
text(x=max(bseq)*.8,y=0.2,labels=paste0("MDE = ",round(MDE_NV1,2)))
text(x=0,y=power(MDE_NV1,NV1/n)*1.05,labels=paste0("power = ",sprintf("%.2f",power(MDE_NV1,NV1/n))))

lines(bseq,beta_NV2,lwd=2,lty='longdash')
abline(h=power(MDE_NV1,NV2/n),lty='dotted')
text(x=0,y=power(MDE_NV1,NV2/n)*1.05,labels=paste0("power = ",sprintf("%.2f",power(MDE_NV1,NV2/n))))

lines(bseq,beta,lwd=2)
abline(h=power(MDE_NV1,V/n),lty='dotted')
text(x=0,y=power(MDE_NV1,V/n)*1.05,labels=paste0("power = ",sprintf("%.2f",power(MDE_NV1,V/n))))
dev.off()
