################################################################################
## Design of Partial Population Experiments 
## with an Application to Spillovers in Tax Compliance
## Guillermo Cruces, Dario Tortarolo, Gonzalo Vazquez-Bare
## Replication code: MDEs and optimal cluster assignment probabilities
################################################################################


rm(list = ls())

################################################################################
## Power function
################################################################################

powerfun <- function(V,tau,alpha){
  beta <- 1-pnorm(tau/sqrt(V)+qnorm(1-alpha/2))+pnorm(tau/sqrt(V)-qnorm(1-alpha/2))
}

################################################################################
## Setup
################################################################################

data_full <- read.csv("C:/Users/wb633983/Documents/git/RR_ARG_2025_258/Bases/data_illustration.csv")
head(data_full)

## Collapse data at cluster level

data_collapsed <- aggregate(data_full$Y,
                  by=list(data_full$cluster),
                  FUN= function(x) c(length(x), mean(x), sd(x)^2))
data_collapsed <- do.call(data.frame,data_collapsed)
data_collapsed <- data_collapsed[,-1]
colnames(data_collapsed) <- c("ng","mug","s2g")
head(data_collapsed)

################################################################################
## Design
################################################################################

pvec <- c(0.2,0.5,0.8)         # Vector of saturations, excluding pure controls
M <- length(pvec)              # Vector length
w_0 <- rep(1/(2*M),M)          # Weights of estimator variances for optimal probabilities (Theorem 2)
w_1 <- rep(1/(2*M),M)          # Weights of estimator variances for optimal probabilities (Theorem 2)
tauseq <- seq(-1,1,by=0.0001)  # Grid of TEs for MDE calculations

################################################################################
## Parameters
################################################################################

rho <- 0.1                     # Intracluster correlation

## Treatment effects on cluster means
# Setting these parameters to zero imposes mu_g(d,t) = mu_g(0,0)

teffect_mu_01 <- 0
teffect_mu_11 <- 0
teffect_mu_02 <- 0
teffect_mu_12 <- 0
teffect_mu_03 <- 0
teffect_mu_13 <- 0

## Treatment effects on cluster variances
# Setting these parameters to zero imposes sigma2_g(d,t) = sigma2_g(0,0)

teffect_s2_01 <- 0
teffect_s2_11 <- 0
teffect_s2_02 <- 0
teffect_s2_12 <- 0
teffect_s2_03 <- 0
teffect_s2_13 <- 0

## Treatment effects on intracluster correlations
# Setting these parameters to zero imposes rho_g(d,t) = rho_g(0,0)

teffect_rho_01 <- 0
teffect_rho_11 <- 0
teffect_rho_02 <- 0
teffect_rho_12 <- 0
teffect_rho_03 <- 0
teffect_rho_13 <- 0

################################################################################
## Prepare data
################################################################################

Results <- matrix(NA,7,9)    # Empty matrix to collect results
Results_R <- matrix(NA,7,9)  # Empty matrix to collect results
column <- 1                  # Column counter to fill result matrices

for (scenario in 1:3){

  # Scenario 1: 8 <= ng
  # Scenario 2: 8 <= ng <= 500
  # Scenario 3: 8 <= ng <= 50
  
  if (scenario==1){
    data <- data_collapsed
  } else if (scenario==2){
    data <- subset(data_collapsed,ng<=500)
  } else if (scenario==3){
    data <- subset(data_collapsed,ng<=50)
  }
  
  n <- sum(data$ng) # Total sample size
  
  for (case in 1:3){
    
    # Case 1: Heterogeneous clusters
    # Case 2: No between-cluster heterogeneity
    # Case 3: No between-cluster heterogeneity and equal cluster sizes
    
    if (case==2){
      #data <- as.data.frame(cbind(data$ng,0,sum((data$ng-1)*data$s2g)/sum(data$ng)))
      data <- as.data.frame(cbind(data$ng,0,var(data_full$Y)))
      #data <- as.data.frame(cbind(data$ng,0,mean(data$s2g)))
    } else if (case==3){
      #data <- as.data.frame(cbind(rep(mean(data$ng),nrow(data)),0,sum((data$ng-1)*data$s2g)/sum(data$ng)))
      data <- as.data.frame(cbind(rep(mean(data$ng),nrow(data)),0,var(data_full$Y)))
      #data <- as.data.frame(cbind(rep(mean(data$ng),nrow(data)),0,mean(data$s2g)))
    }
    
    colnames(data) <- c("ng","mug","s2g")
    
    print(head(data))
    
    ################################################################################
    ## Impute other parameters
    ################################################################################
    
    data$m2_0 <- (data$mug-mean(data$mug))^2
    
    data$m2_01 <- data$m2_0*(1+teffect_mu_01)
    data$m2_11 <- data$m2_0*(1+teffect_mu_11)
    data$m2_02 <- data$m2_0*(1+teffect_mu_02)
    data$m2_12 <- data$m2_0*(1+teffect_mu_12)
    data$m2_03 <- data$m2_0*(1+teffect_mu_03)
    data$m2_13 <- data$m2_0*(1+teffect_mu_13)
    
    data$s2g_01 <- data$s2g*(1+teffect_s2_01)
    data$s2g_11 <- data$s2g*(1+teffect_s2_11)
    data$s2g_02 <- data$s2g*(1+teffect_s2_02)
    data$s2g_12 <- data$s2g*(1+teffect_s2_12)
    data$s2g_03 <- data$s2g*(1+teffect_s2_03)
    data$s2g_13 <- data$s2g*(1+teffect_s2_13)
    
    data$rhog <- rho
    data$rhog_01 <- data$rhog*(1+teffect_rho_01)
    data$rhog_11 <- data$rhog*(1+teffect_rho_11)
    data$rhog_02 <- data$rhog*(1+teffect_rho_02)
    data$rhog_12 <- data$rhog*(1+teffect_rho_12)
    data$rhog_03 <- data$rhog*(1+teffect_rho_03)
    data$rhog_13 <- data$rhog*(1+teffect_rho_13)

    ################################################################################
    ## Calculate variance components from Equation (8)
    ################################################################################
    
    B0 <- sum(data$ng*(data$s2g*(1+data$rhog*(data$ng-1))+data$ng*data$m2_0))
    
    B01 <- sum(data$ng*(data$s2g_01*(1+data$rhog_01*(1-pvec[1])*(data$ng-1))+data$m2_01*(1+(1-pvec[1])*(data$ng-1)))/(1-pvec[1]))
    B11 <- sum(data$ng*(data$s2g_11*(1+data$rhog_11*pvec[1]*(data$ng-1))+data$m2_11*(1+pvec[1]*(data$ng-1)))/pvec[1])
    B1 <- w_0[1]*B01 + w_1[1]*B11
    
    B02 <- sum(data$ng*(data$s2g_02*(1+data$rhog_02*(1-pvec[2])*(data$ng-1))+data$m2_02*(1+(1-pvec[2])*(data$ng-1)))/(1-pvec[2]))
    B12 <- sum(data$ng*(data$s2g_12*(1+data$rhog_12*pvec[2]*(data$ng-1))+data$m2_12*(1+pvec[2]*(data$ng-1)))/pvec[2])
    B2 <- w_0[2]*B02 + w_1[2]*B12
    
    B03 <- sum(data$ng*(data$s2g_03*(1+data$rhog_03*(1-pvec[3])*(data$ng-1))+data$m2_03*(1+(1-pvec[3])*(data$ng-1)))/(1-pvec[3]))
    B13 <- sum(data$ng*(data$s2g_13*(1+data$rhog_13*pvec[3]*(data$ng-1))+data$m2_13*(1+pvec[3]*(data$ng-1)))/pvec[3])
    B3 <- w_0[3]*B03 + w_1[3]*B13
    
    ################################################################################
    ## Calculate optimal cluster probabilities from Theorem 2
    ################################################################################
    
    q0 <- sqrt(B0) / (sqrt(B0) + sqrt(B1) + sqrt(B2) + sqrt(B3))
    q1 <- sqrt(B1) / (sqrt(B0) + sqrt(B1) + sqrt(B2) + sqrt(B3))
    q2 <- sqrt(B2) / (sqrt(B0) + sqrt(B1) + sqrt(B2) + sqrt(B3))
    q3 <- sqrt(B3) / (sqrt(B0) + sqrt(B1) + sqrt(B2) + sqrt(B3))
    
    qprobs <- c(q0,q1,q2,q3)
    
    ################################################################################
    ## Calculate variances with optimal q
    ################################################################################
    
    V01 <- B01/(n^2*q1) + B0/(n^2*q0)
    V11 <- B11/(n^2*q1) + B0/(n^2*q0)
    V02 <- B02/(n^2*q2) + B0/(n^2*q0)
    V12 <- B12/(n^2*q2) + B0/(n^2*q0)
    V03 <- B03/(n^2*q3) + B0/(n^2*q0)
    V13 <- B13/(n^2*q3) + B0/(n^2*q0)
    
    Vars <- c(V01,V02,V03)
    
    ################################################################################
    ## Calculate power and MDE with optimal q
    ################################################################################
    
    beta_01 <- powerfun(V01,tauseq,0.05)
    MDE01_1 <- tauseq[min(which(beta_01<=0.8))-1]
    MDE01_2 <- tauseq[max(which(beta_01<=0.8))+1]
    MDE_01 <- min(abs(MDE01_1),abs(MDE01_2))
    
    beta_11 <- powerfun(V11,tauseq,0.05)
    MDE11_1 <- tauseq[min(which(beta_11<=0.8))-1]
    MDE11_2 <- tauseq[max(which(beta_11<=0.8))+1]
    MDE_11 <- min(abs(MDE11_1),abs(MDE11_2))
    
    beta_02 <- powerfun(V02,tauseq,0.05)
    MDE02_1 <- tauseq[min(which(beta_02<=0.8))-1]
    MDE02_2 <- tauseq[max(which(beta_02<=0.8))+1]
    MDE_02 <- min(abs(MDE02_1),abs(MDE02_2))
    
    beta_12 <- powerfun(V12,tauseq,0.05)
    MDE12_1 <- tauseq[min(which(beta_12<=0.8))-1]
    MDE12_2 <- tauseq[max(which(beta_12<=0.8))+1]
    MDE_12 <- min(abs(MDE12_1),abs(MDE12_2))
    
    beta_03 <- powerfun(V03,tauseq,0.05)
    MDE03_1 <- tauseq[min(which(beta_03<=0.8))-1]
    MDE03_2 <- tauseq[max(which(beta_03<=0.8))+1]
    MDE_03 <- min(abs(MDE03_1),abs(MDE03_2))
    
    beta_13 <- powerfun(V13,tauseq,0.05)
    MDE13_1 <- tauseq[min(which(beta_13<=0.8))-1]
    MDE13_2 <- tauseq[max(which(beta_13<=0.8))+1]
    MDE_13 <- min(abs(MDE13_1),abs(MDE13_2))
    
    MDEs <- c(MDE_01,MDE_02,MDE_03)
    
    ################################################################################
    ## Calculate restricted cluster probabilities
    ################################################################################
    
    L <- 25061                           # Total number of letters to be sent
    q1_R <- (L/n)*(2*B11/(B02+2*B11))
    q2_R <- B02*q1_R/B11
    q3_R <- q1_R
    q0_R <- 1 - q1_R - q2_R - q3_R
    
    qprobs_R <- c(q0_R,q1_R,q2_R,q3_R)
    
    ################################################################################
    ## Calculate variances with restricted q
    ################################################################################
    
    V01_R <- B01/(n^2*q1_R) + B0/(n^2*q0_R)
    V11_R <- B11/(n^2*q1_R) + B0/(n^2*q0_R)
    V02_R <- B02/(n^2*q2_R) + B0/(n^2*q0_R)
    V12_R <- B12/(n^2*q2_R) + B0/(n^2*q0_R)
    V03_R <- B03/(n^2*q3_R) + B0/(n^2*q0_R)
    V13_R <- B13/(n^2*q3_R) + B0/(n^2*q0_R)
    
    Vars <- c(V01_R,V02_R,V03_R)
    
    ################################################################################
    ## Calculate power and MDE with restricted q
    ################################################################################
    
    beta_01_R <- powerfun(V01_R,tauseq,0.05)
    MDE01_1_R <- tauseq[min(which(beta_01_R<=0.8))-1]
    MDE01_2_R <- tauseq[max(which(beta_01_R<=0.8))+1]
    MDE_01_R  <- min(abs(MDE01_1_R),abs(MDE01_2_R))
    
    beta_11_R <- powerfun(V11_R,tauseq,0.05)
    MDE11_1_R <- tauseq[min(which(beta_11_R<=0.8))-1]
    MDE11_2_R <- tauseq[max(which(beta_11_R<=0.8))+1]
    MDE_11_R  <- min(abs(MDE11_1_R),abs(MDE11_2_R))
    
    beta_02_R <- powerfun(V02_R,tauseq,0.05)
    MDE02_1_R <- tauseq[min(which(beta_02_R<=0.8))-1]
    MDE02_2_R <- tauseq[max(which(beta_02_R<=0.8))+1]
    MDE_02_R  <- min(abs(MDE02_1_R),abs(MDE02_2_R))
    
    beta_12_R <- powerfun(V12_R,tauseq,0.05)
    MDE12_1_R <- tauseq[min(which(beta_12_R<=0.8))-1]
    MDE12_2_R <- tauseq[max(which(beta_12_R<=0.8))+1]
    MDE_12_R  <- min(abs(MDE12_1_R),abs(MDE12_2_R))
    
    beta_03_R <- powerfun(V03,tauseq,0.05)
    MDE03_1_R <- tauseq[min(which(beta_03<=0.8))-1]
    MDE03_2_R <- tauseq[max(which(beta_03<=0.8))+1]
    MDE_03_R  <- min(abs(MDE03_1),abs(MDE03_2))
    
    beta_13_R <- powerfun(V13_R,tauseq,0.05)
    MDE13_1_R <- tauseq[min(which(beta_13_R<=0.8))-1]
    MDE13_2_R <- tauseq[max(which(beta_13_R<=0.8))+1]
    MDE_13_R  <- min(abs(MDE13_1_R),abs(MDE13_2_R))
    
    MDEs_R <- c(MDE_01_R,MDE_02_R,MDE_03_R)
    
    ################################################################################
    ## Collect and export results
    ################################################################################
    
    Results[,column] <- c(qprobs,MDEs)
    Results_R[,column] <- c(qprobs_R,MDEs_R)
    
    rownames(Results) <- c("q0","q1","q2","q3","MDE_01","MDE_02","MDE_03")
    rownames(Results_R) <- c("q0","q1","q2","q3","MDE_01","MDE_02","MDE_03")
    
    column <- column + 1
    
  }
  
}

round(Results_R,3)
round(Results,3)

################################################################################
## Bottom part of Table 1: Scenario Statistics
################################################################################

Sumstats <- matrix(NA,6,3)
Sumstats[1,] <- c(sum(data_collapsed$ng),sum(subset(data_collapsed,ng<=500)$ng),sum(subset(data_collapsed,ng<=50)$ng))
Sumstats[2,] <- c(nrow(data_collapsed),nrow(subset(data_collapsed,ng<=500)),nrow(subset(data_collapsed,ng<=50)))
Sumstats[3,] <- c(min(data_collapsed$ng),min(subset(data_collapsed,ng<=500)$ng),min(subset(data_collapsed,ng<=50)$ng))
Sumstats[4,] <- c(max(data_collapsed$ng),max(subset(data_collapsed,ng<=500)$ng),max(subset(data_collapsed,ng<=50)$ng))
Sumstats[5,] <- c(mean(data_collapsed$ng),mean(subset(data_collapsed,ng<=500)$ng),mean(subset(data_collapsed,ng<=50)$ng))
Sumstats[6,] <- c(sd(data_collapsed$ng),sd(subset(data_collapsed,ng<=500)$ng),sd(subset(data_collapsed,ng<=50)$ng))
round(Sumstats,1)

