** Open db with baseline payments for 2019
use data_baseline, clear

set seed 9221214

global G1 = 1100 // Number of groups with T=1
global G2 = 680  // Number of groups with T=2
global G3 = 1100 // Number of groups with T=3

********************************************************************************
** Group assignment
********************************************************************************

preserve
** Keep one observation per group (dataset at the group level)
collapse gcat pago_todas pago_alguna pago_seis, by(g)

gen rsort = runiform() // Randomly sort groups
sort rsort

** Assign treatment to groups

gen T = 1 if _n<=$G1
replace T = 2 if _n>$G1 & _n<=$G1+$G2
replace T = 3 if _n>$G1+$G2 & _n<=$G1+$G2+$G3
replace T = 0 if T==.
drop rsort

** Resulting sample sizes

tab T

** By group size category 

tab T gcat

** Balanced?
forv g=1(1)3 {
  gen B`g'=0 if T==0
  replace B`g'=1 if T==`g'
  reg pago_todas B`g'
  drop B`g'
}

sort g
save assignment_group.dta, replace
restore

********************************************************************************
** Individual assignment
********************************************************************************

** Merge group-level assignment info

merge m:1 g using assignment_group
drop _merge

tab T
tab T if tag

** Assign treatment within groups

gen prob = 0.2*(T==1) + 0.5*(T==2) + 0.8*(T==3)                                     // Probability of treatment in each group 
gen coin = runiform()<0.5                                                           // Rounding: up or down
gen rsort = runiform()                                                              // Randomly sort observations within group
bysort g (rsort): gen D = (_n<=floor(_N*prob))*coin + (_n<=ceil(_N*prob))*(1-coin)  // Assign individual treatment
tab T D, row
drop coin rsort

** Observed proportion of treated

bysort g: egen sD = sum(D)
gen prop = sD/gsize
table T, c(m prop sd prop)
hist prop if tag==1 & prop>0, title("Observed proportion of treated")

** Difference between probability and observed proportion

gen dif_prop = prob - prop
sum dif_prop
hist dif_prop, title("Difference expected / observed proportion")

** Resulting number of treated and controls

tab T if tag

tab D
tab T D, row

sort g id

** Balanced?
reg pago_todas D
reg pago_alguna D
reg pago_seis D

reg pago_todas D if T==1
reg pago_todas D if T==2
reg pago_todas D if T==3

reg pago_alguna D if T==1
reg pago_alguna D if T==2
reg pago_alguna D if T==3

reg pago_seis D if T==1
reg pago_seis D if T==2
reg pago_seis D if T==3


********************************************************************************
** Descriptive statistics
********************************************************************************

codebook g
sum gsize if tag==1
sum pago*

*Table A1: Sample sizes (N clusters, treated, control --> by T)
unique g, by(T)
tab T D

*Table A2: Descriptive statistics in 2019 (baseline year)
mat Stats = J(3,5,.)
local row = 1
foreach var of varlist pago_todas-pago_seis{
	qui duplicates report g
	mat Stats[`row',1] = r(unique_value)
	qui sum `var'
	mat Stats[`row',2] = r(N)
	mat Stats[`row',3] = r(mean)
	mat Stats[`row',4] = r(sd)
	qui loneway `var' g
	mat Stats[`row',5] = r(rho)
	local ++row
} 
matlist Stats
preserve
	drop _all
	svmat Stats
	export delimited "$Results\Stats.csv", replace
restore



** Final dataset with treatment assignment

keep id2 id nro_cuenta T D
rename D tratado
label define tratado 1 "tratado"  0 "control"
label values tratado tratado
label var tratado "Asignacion a nivel de cuenta"
label var T "Asignacion a nivel de grupo"
sort id2
save randomization_final.dta, replace

